/*
  File name        : core.h
  Date of creation : 10/2/2008
  Version          : 1.0
  Copyright        : Soft'n'Design
  Author           : Laurent Mazet <mazet@softndesign.org>

  Description      : Miscellaenous functions on core/thread management

  History          :
  - initial version from tools.h
*/

#ifndef __CORE_H__
#define __CORE_H__

#include <sys/cdefs.h>

__BEGIN_DECLS

/**
   Declare a parameter or a variable unused.
*/
#define UNUSED __attribute__ ((unused))

/**
   Count number of CPU

   @return number of CPU
*/
int get_cpu_number (void);

/* Thread management */

/**
   Set realtime priority

   @param prio priority
   @return error code (from pthread_setschedparam)
*/
int enable_realtime (int prio);

/**
   Set cpu (core) affinity

   @param cpu_list is a char string describing cpu affinity when core ids are separated by plus (+), comma (,) or space ( )
   @return error code (from pthread_setaffinity_np)
*/
int cpu_setaffinity (const char *cpu_list);

/**
   Compute the difference between two cycle counters

   @param tsc1 initial cycle counter
   @param tsc1 last cycle counter
   @return cycle counter difference
*/
unsigned long long tsc2diff (unsigned long long tsc1, unsigned long long tsc2);

/**
   Evaluation a number of cycles in milisecond

   @param delta_tsc number of cycles
   @return number of miliseconds
*/
float tsc2ms (unsigned long long delta_tsc);

__END_DECLS

#endif /* __CORE_H__ */


/* vi:set tabstop=4 expandtab shiftwidth=4: this line set vi mode */
