/*
  File name        : queue.c
  Date of creation : 25/01/2010
  Version          : 1.0
  Copyright        : Soft'n'Design
  Author           : Laurent Mazet <mazet@softndesign,org>

  Description      : Queue management

  History          :
  - initial version
*/

#ifndef __QUEUE_H__
#define __QUEUE_H__

#include <pthread.h>

__BEGIN_DECLS

typedef enum {
    QUEUE_INACTIVE = 0,
    QUEUE_INIT,
    QUEUE_WAITING,
    QUEUE_LOADING,
    QUEUE_LOADED,
    QUEUE_ACTIVE,
    QUEUE_BUSY,
    QUEUE_TERMINATING
} queue_tastk_state_t;

typedef struct {
    queue_tastk_state_t state;
    int (*task) (void *params);
    void *params;
    pthread_rwlock_t *wait_rwlock;
} queue_sub_task_t;

typedef struct {
    pthread_t thread;
    queue_tastk_state_t state;
    long long int nb_calls;
    long long int nb_execs;
} queue_task_info_t;

typedef struct {
    pthread_rwlock_t wait_rwlock;
} queue_sub_task_info_t;

typedef struct {
    char *name;
    int nb_sub_tasks;
    int nb_sub_queues;
    int priority;
    char *cpu_list;
    void (*init) (void);
    queue_task_info_t *sub_queue_list;
    queue_sub_task_t *sub_task_list;
    pthread_mutex_t load_mutex;
    pthread_mutex_t exec_mutex;
    pthread_cond_t wakeup_cond;
    /* CPU occupancy evaluation */
    unsigned long long *nb_cycles;
    int *nb_calls;
    unsigned long long *nb_cycles_last;
    int *nb_calls_last;
    unsigned long long *nb_cycles_global;
    int *nb_calls_global;
} queue_task_t;


queue_task_t *create_queue_task (const char *name, int nb_sub_tasks, int priority,
                                 int nb_sub_queues, const char *cpu_list,
                                 void (*init) (void));

queue_sub_task_info_t *queue_init (queue_task_t *t);

int queue_load (queue_task_t *t, queue_sub_task_info_t *st,
                int (*task) (void *), void *params);

int queue_end (queue_task_t *t, queue_sub_task_info_t *st);

void queue_stat (queue_task_t *t, int condensed);

__END_DECLS

#endif /* __QUEUE_H__ */

/* vi:set tabstop=4 expandtab shiftwidth=4: this line set vi mode */
